// VERSION DoubleMetaphone Class 1.01
//
// DESCRIPTION
//
//   This class implements a "sounds like" algorithm developed
//   by Lawrence Philips which he published in the June, 2000 issue
//   of C/C++ Users Journal.  Double Metaphone is an improved
//   version of Philips' original Metaphone algorithm.
//
// COPYRIGHT
//
//   Copyright 2001, Stephen Woodbridge <woodbri@swoodbridge.com>
//   All rights reserved.
//
//   http://swoodbridge.com/DoubleMetaPhone/
//
//   This Xbase++ translation is based heavily on the C implementation
//   by Maurice Aubrey <maurice@hevanet.com>, which in turn
//   is based heavily on the C++ implementation by
//   Lawrence Philips and incorporates several bug fixes courtesy
//   of Kevin Atkinson <kevina@users.sourceforge.net>.
//
//   This module is free software; you may redistribute it and/or
//   modify it under the same terms as Perl itself.
//
// CONTRIBUTIONS
//
//   17-May-2002 Geoff Caplan  http://www.advantae.com
//     Bug fix: added code to return class object which I forgot to do
//     Created a functional callable version instead of the class version
//     which is faster if you are calling this a lot.
//
//   06-Sep-2003 Roger Donnay http://donnay-software.com
//     Converted to Xbase++
//
// ------------------------------------------------------------------

CLASS DoubleMetaPhone

   EXPORTED:

   VAR original
   VAR primary
   VAR secondary
   VAR length
   VAR last
   VAR current

   METHOD Init, DoubleMetaPhone, StringAt, IsVowel, SlavoGermanic

ENDCLASS

* ----------------

METHOD DoubleMetaPhone:init()

   ::original := ''
   ::primary := ''
   ::secondary := ''
   ::length := 0
   ::last := 0
   ::current := 0

RETURN self

* -----------------

METHOD DoubleMetaPhone:DoubleMetaPhone( cString )

   LOCAL cChar

   ::primary := ''
   ::secondary := ''
   ::current := 1
   ::length := Len(cString)
   ::last := ::length
   ::original := cString + "     "
   ::original := Upper(::original)

   // skip this at beginning of word

   IF ::StringAt(::original, 1, 2, { 'GN', 'KN', 'PN', 'WR', 'PS' } )
      ::current++
   ENDIF

   // Initial 'X' is pronounced 'Z' e.g. 'Xavier'
   IF (SubStr(::original, 1, 1) == 'X')
      ::primary += "S"                          // 'Z' maps to 'S'
      ::secondary += "S"
      ::current++
   ENDIF

   DO WHILE Len(::primary) < 4 .OR. Len(::secondary) < 4

      IF ::current > ::length
         EXIT
      ENDIF

      cChar := SubStr( ::original, ::current, 1 )

      DO CASE

    /*
    CASE cChar $ 'AEIOUY'
      IF ::current == 1
        // all init vowels now map to 'A'
        ::primary += 'A'
        ::secondary += 'A'
      ENDIF
      ::current++
    */

      CASE cChar = 'A'
         IF ::current == 1
            ::primary += 'A'
            ::secondary += 'A'
         ENDIF
         ::current++

      CASE cChar = 'E'
         IF ::current == 1
            ::primary += 'E'
            ::secondary += 'E'
         ENDIF
         ::current++

      CASE cChar = 'I'
         IF ::current == 1
            ::primary += 'I'
            ::secondary += 'I'
         ENDIF
         ::current++

      CASE cChar = 'O'
         IF ::current == 1
            ::primary += 'O'
            ::secondary += 'O'
         ENDIF
         ::current++

      CASE cChar = 'U'
         IF ::current == 1
            ::primary += 'U'
            ::secondary += 'U'
         ENDIF
         ::current++

      CASE cChar = 'Y'
         IF ::current == 1
            ::primary += 'Y'
            ::secondary += 'Y'
         ENDIF
         ::current++

      CASE cChar == 'B'
         // '-mb', e.g. "dumb", already skipped over ...
         ::primary   += 'B'
         ::secondary += 'P'

         IF Substr(::original, ::current + 1, 1) == 'B'
            ::current += 2
         ELSE
            ::current += 1
         ENDIF

      CASE cChar == ''
         ::primary   += 'S'
         ::secondary += 'S'
         ::current++

      CASE cChar == 'C'

         // various gremanic
         IF ((::current > 2 ) ;
               .AND. !::IsVowel(::original, ::current - 2) ;
               .AND. ::StringAt(::original, ::current - 1, 3, {"ACH"}) ;
               .AND. ((SubStr(::original, ::current + 2, 1) != 'I') ;
               .AND. ((SubStr(::original, ::current + 2, 1) != 'E') ;
               .OR. ::StringAt(::original, ::current - 2, 6, {"BACHER", "MACHER"}))))

            ::primary   += 'K'
            ::secondary += 'K'
            ::current += 2
            LOOP
         ENDIF

         // special case 'caesar'
         IF (::current == 1 ;
               .AND. ::StringAt(::original, ::current, 6, {"CAESAR"}))
            ::primary   += 'S'
            ::secondary += 'S'
            ::current += 2
            LOOP
         ENDIF

         // italian 'chianti'
         IF (::StringAt(::original, ::current, 4, {"CHIA"}))
            ::primary   += 'K'
            ::secondary += 'K'
            ::current += 2
            LOOP
         ENDIF

         IF (::StringAt(::original, ::current, 2, {"CH"}))

            // find 'michael'
            IF (::current > 1 ;
                  .AND. ::StringAt(::original, ::current, 4, {"CHAE"}))
               ::primary   += 'K'
               ::secondary += 'X'
               ::current += 2
               LOOP
            ENDIF

            // greek roots e.g. 'chemistry', 'chorus'
            IF (::current == 1 ;
                  .AND. (::StringAt(::original, ::current + 1, 5, {"HARAC", "HARIS"} ) ;
                  .OR. ::StringAt(::original, ::current + 1, 3, {"HOR", "HYM", "HIA", "HEM"})) ;
                  .AND. !::StringAt(::original, 1, 5, {"CHORE"}))
               ::primary   += 'K'
               ::secondary += 'K'
               ::current += 2
               LOOP
            ENDIF

            // germanic, greek, or otherwise 'ch' for 'kh' sound
            IF ((::StringAt(::original, 1, 4, {"VAN ", "VON "}) ;
                  .OR. ::StringAt(::original, 1, 3, {"SCH"})) ;
                  ;                             // 'architect' but not 'arch', orchestra', 'orchid' ;
                  .OR. ::StringAt(::original, ::current - 2, 6, {"ORCHES", "ARCHIT", "ORCHID"}) ;
                  .OR. ::StringAt(::original, ::current + 2, 1, {"T", "S"}) ;
                  .OR. ((::StringAt(::original, ::current - 1, 1, {"A","O","U","E"}) ;
                  .OR. (::current == 1)) ;
                  ;                             // e.g. 'wachtler', 'weschsler', but not 'tichner' ;
                  .AND. ::StringAt(::original, ::current + 2, 1, {"L","R","N","M","B","H","F","V","W"," "})))
               ::primary   += 'K'
               ::secondary += 'K'
            ELSE
               IF (::current > 1)
                  IF (::StringAt(::original, 1, 2, {"MC"}))
                     // e.g. 'McHugh'
                     ::primary   += 'K'
                     ::secondary += 'K'
                  ELSE
                     ::primary   += 'X'
                     ::secondary += 'K'
                  ENDIF
               ELSE
                  ::primary   += 'X'
                  ::secondary += 'X'
               ENDIF
            ENDIF
            ::current += 2
            LOOP
         ENDIF

         // e.g. 'czerny'
         IF (::StringAt(::original, ::current, 2, {"CZ"}) ;
               .AND. !::StringAt(::original, ::current -2, 4, {"WICZ"}))
            ::primary   += 'S'
            ::secondary += 'X'
            ::current += 2
            LOOP
         ENDIF

         // e.g. 'focaccia'
         IF (::StringAt(::original, ::current + 1, 3, {"CIA"}))
            ::primary   += 'X'
            ::secondary += 'X'
            ::current += 3
            LOOP
         ENDIF

         // double 'C', but not McClellan'
         IF (::StringAt(::original, ::current, 2, {"CC"}) ;
               .AND. !((::current == 2) ;
               .AND. (SubStr(::original, 1, 1) == 'M')))
            // 'bellocchio' but not 'bacchus'
            IF (::StringAt(::original, ::current + 2, 1, {"I","E","H"}) ;
                  .AND. !::StringAt(::original, ::current + 2, 2, {"HU"}))
               // 'accident', 'accede', 'succeed'
               IF (((::current == 2) ;
                     .AND. (SubStr(::original, ::current - 1, 1) == 'A')) ;
                     .OR. ::StringAt(::original, ::current - 1, 5, {"UCCEE", "UCCES"}))
                  ::primary   += "KS"
                  ::secondary += "KS"
                  // 'bacci', 'bertucci', other italian
               ELSE
                  ::primary   += "X"
                  ::secondary += "X"
               ENDIF
               ::current += 3
               LOOP
            ELSE
               // Pierce's rule
               ::primary   += "K"
               ::secondary += "K"
               ::current += 2
               LOOP
            ENDIF
         ENDIF

         IF (::StringAt(::original, ::current, 2, {"CK","CG","CQ"}))
            ::primary   += "K"
            ::secondary += "K"
            ::current += 2
            LOOP
         ENDIF

         IF (::StringAt(::original, ::current, 2, {"CI","CE","CY"}))
            // italian vs. english
            IF (::StringAt(::original, ::current, 3, {"CIO","CIE","CIA"}))
               ::primary   += "S"
               ::secondary += "X"
            ELSE
               ::primary   += "S"
               ::secondary += "S"
            ENDIF
            ::current += 2
            LOOP
         ENDIF
         // else
         ::primary   += "K"
         ::secondary += "K"

         // name sent in 'mac caffrey', 'mac gregor'
         IF (::StringAt(::original, ::current + 1, 2, {" C"," Q"," G"}))
            ::current += 3
         ELSE
            IF (::StringAt(::original, ::current + 1, 1, {"C","K","Q"}) ;
                  .AND. !::StringAt(::original, ::current + 1, 2, {"CE","CI"}))
               ::current += 2
            ELSE
               ::current += 1
            ENDIF
         ENDIF

      CASE cChar == 'D'
         if (::StringAt(::original, ::current, 2, {"DG"}))
            if (::StringAt(::original, ::current + 2, 1, {"I","E","Y"}))
               // e.g. 'edge'
               ::primary   += "J"
               ::secondary += "J"
               ::current += 3
            else
               // e.g. 'edgar'
               ::primary   += "TK"
               ::secondary += "TK"
               ::current += 2
            endif
            LOOP
         endif

         if (::StringAt(::original, ::current, 2, {"DT","DD"}))
            ::primary   += "T"
            ::secondary += "T"
            ::current += 2
            LOOP
         else
            // else
            ::primary   += "D"
            ::secondary += "T"
            ::current += 1
         endif

      CASE cChar == 'F'
         if (SubStr(::original, ::current + 1, 1) == 'F')
            ::current += 2
         else
            ::current += 1
         endif
         ::primary   += "F"
         ::secondary += "F"

      case cChar == 'G'
         if (substr(::original, ::current + 1, 1) == 'H')
            if ((::current > 1) ;
                  .AND. !::IsVowel(::original, ::current - 1))
               ::primary   += "K"
               ::secondary += "K"
               ::current += 2
               loop
            endif

            if (::current < 4)
               // 'ghislane', 'ghiradelli'
               if (::current == 1)
                  if (substr(::original, ::current + 2, 1) == 'I')
                     ::primary   += "J"
                     ::secondary += "J"
                  else
                     ::primary   += "K"
                     ::secondary += "K"
                  endif
                  ::current += 2
                  loop
               endif
            endif

            // Parker's rule (with some further refinements) - e.g. 'hugh'
            if (((::current > 2) ;
                  .AND. ::StringAt(::original, ::current - 2, 1, {"B","H","D"})) ;
                  ;                             // e.g. 'bough' ;
                  .OR. ((::current > 3) ;
                  .AND. ::StringAt(::original, ::current - 3, 1, {"B","H","D"})) ;
                  ;                             // e.g. 'broughton' ;
                  .OR. ((::current > 4) ;
                  .AND. ::StringAt(::original, ::current - 4, 1, {"B","H"})))
               ::current += 2
               LOOP
            else
               // e.g. 'laugh', 'McLaughlin', 'cough', 'gough', 'rough', 'tough'
               if ((::current > 3) ;
                     .AND. (substr(::original, ::current - 1, 1) == 'U') ;
                     .AND. ::StringAt(::original, ::current - 3, 1, {"C","G","L","R","T"}))
                  ::primary   += "F"
                  ::secondary += "F"
               elseif ((::current > 1) ;
                     .AND. substr(::original, ::current - 1, 1) != 'I')
                  ::primary   += "K"
                  ::secondary += "K"
               endif
               ::current += 2
               LOOP
            endif
         endif

         if (substr(::original, ::current + 1, 1) == 'N')
            if ((::current == 2) .AND. ::IsVowel(::original, 1) ;
                  .AND. !::SlavoGermanic(::original))
               ::primary   += "KN"
               ::secondary += "N"
            else
               // not e.g. 'cagney'
               if (!::StringAt(::original, ::current + 2, 2, {"EY"}) ;
                     .AND. (substr(::original, ::current + 1) != "Y") ;
                     .AND.  !::SlavoGermanic(::original))
                  ::primary   += "N"
                  ::secondary += "KN"
               else
                  ::primary   += "KN"
                  ::secondary += "KN"
               endif
            endif
            ::current += 2
            LOOP
         endif

         // 'tagliaro'
         if (::StringAt(::original, ::current + 1, 2, {"LI"}) ;
               .AND. !::SlavoGermanic(::original))
            ::primary   += "KL"
            ::secondary += "L"
            ::current += 2
            LOOP
         endif

         // -ges-, -gep-, -gel- at beginning
         if ((::current == 1) ;
               .AND. ((substr(::original, ::current + 1, 1) == 'Y') ;
               .OR. ::StringAt(::original, ::current + 1, 2, ;
               {"ES","EP","EB","EL","EY","IB","IL","IN","IE", ;
               "EI","ER"})))
            ::primary   += "K"
            ::secondary += "J"
            ::current += 2
            LOOP
         endif

         // -ger-, -gy-
         if ((::StringAt(::original, ::current + 1, 2, {"ER"}) ;
               .OR. (substr(::original, ::current + 1, 1) == 'Y')) ;
               .AND. !::StringAt(::original, 1, 6, {"DANGER","RANGER","MANGER"}) ;
               .AND. !::StringAt(::original, ::current -1, 1, {"E", "I"}) ;
               .AND. !::StringAt(::original, ::current -1, 3, {"RGY","OGY"}))
            ::primary   += "K"
            ::secondary += "J"
            ::current += 2
            LOOP
         endif

         // italian e.g. 'biaggi'
         if (::StringAt(::original, ::current + 1, 1, {"E","I","Y"}) ;
               .OR. ::StringAt(::original, ::current -1, 4, {"AGGI","OGGI"}))
            // obvious germanic
            if ((::StringAt(::original, 1, 4, {"VAN ", "VON "}) ;
                  .OR. ::StringAt(::original, 1, 3, {"SCH"})) ;
                  .OR. ::StringAt(::original, ::current + 1, 2, {"ET"}))
               ::primary   += "K"
               ::secondary += "K"
            else
               // always soft if french ending
               if (::StringAt(::original, ::current + 1, 4, {"IER "}))
                  ::primary   += "J"
                  ::secondary += "J"
               else
                  ::primary   += "J"
                  ::secondary += "K"
               endif
            endif
            ::current += 2
            LOOP
         endif

         if (substr(::original, ::current +1, 1) == 'G')
            ::current += 2
         else
            ::current += 1
         endif
         ::primary   += 'K'
         ::secondary += 'K'

      case cChar == 'H'
         // only keep if first & before vowel or btw. 2 vowels
         if (((::current == 1) .OR. ::IsVowel(::original, ::current - 1)) ;
               .AND. ::IsVowel(::original, ::current + 1))
            ::primary   += 'H'
            ::secondary += 'H'
            ::current += 2
         else
            ::current += 1
         endif

      case cChar == 'J'
         // obvious spanish, 'jose', 'san jacinto'
         if (::StringAt(::original, ::current, 4, {"JOSE"}) ;
               .OR. ::StringAt(::original, 1, 4, {"SAN "}))
            if (((::current == 1) ;
                  .AND. (substr(::original, ::current + 4, 1) == ' ')) ;
                  .OR. ::StringAt(::original, 1, 4, {"SAN "}))
               ::primary   += 'H'
               ::secondary += 'H'
            else
               ::primary   += "J"
               ::secondary += 'H'
            endif
            ::current += 1
            LOOP
         endif

         if ((::current == 1) ;
               .AND. !::StringAt(::original, ::current, 4, {"JOSE"}))
            ::primary   += 'J'                  // Yankelovich/Jankelowicz
            ::secondary += 'A'
         else
            // spanish pron. of .e.g. 'bajador'
            if (::IsVowel(::original, ::current - 1) ;
                  .AND. !::SlavoGermanic(::original) ;
                  .AND. ((substr(::original, ::current + 1, 1) == 'A') ;
                  .OR. (substr(::original, ::current + 1, 1) == 'O')))
               ::primary   += "J"
               ::secondary += "H"
            else
               if (::current == ::last)
                  ::primary   += "J"
                  ::secondary += ""
               else
                  if (!::StringAt(::original, ::current + 1, 1, {"L","T","K","S","N","M","B","Z"}) ;
                        .AND. !::StringAt(::original, ::current - 1, 1, {"S","K","L"}))
                     ::primary   += "J"
                     ::secondary += "J"
                  endif
               endif
            endif
         endif

         if (substr(::original, ::current + 1, 1) == 'J') // it could happen
            ::current += 2
         else
            ::current += 1
         endif

      case cChar == 'K'
         if (substr(::original, ::current + 1, 1) == 'K')
            ::current += 2
         else
            ::current += 1
         endif
         ::primary   += "K"
         ::secondary += "K"

      case cChar == 'L'
         if (substr(::original, ::current + 1, 1) == 'L')
            // spanish e.g. 'cabrillo', 'gallegos'
            if (((::current == (::length - 3)) ;
                  .AND. ::StringAt(::original, ::current - 1, 4, {"ILLO","ILLA","ALLE"})) ;
                  .OR. ((::StringAt(::original, ::last-1, 2, {"AS","OS"}) ;
                  .OR. ::StringAt(::original, ::last, 1, {"A","O"})) ;
                  .AND. ::StringAt(::original, ::current - 1, 4, {"ALLE"})))
               ::primary   += "L"
               ::secondary += ""
               ::current += 2
               LOOP
            endif
            ::current += 2
         else
            ::current += 1
         endif
         ::primary   += "L"
         ::secondary += "L"

      case cChar == 'M'
         if ((::StringAt(::original, ::current - 1, 3, {"UMB"}) ;
               .AND. (((::current + 1) == ::last) ;
               .OR. ::StringAt(::original, ::current + 2, 2, {"ER"}))) ;
               ;                                // 'dumb', 'thumb' ;
               .OR. (substr(::original, ::current + 1, 1) == 'M'))
            ::current += 2
         else
            ::current += 1
         endif
         ::primary   += "M"
         ::secondary += "M"

      case cChar == 'N'
         if (substr(::original, ::current + 1, 1) == 'N')
            ::current += 2
         else
            ::current += 1
         endif
         ::primary   += "N"
         ::secondary += "N"

      case cChar == ''
         ::current += 1
         ::primary   += "N"
         ::secondary += "N"

      case cChar == 'P'
         if (substr(::original, ::current + 1, 1) == 'H')
            ::current += 2
            ::primary   += "F"
            ::secondary += "F"
            LOOP
         endif

         // also account for "campbell" and "raspberry"
         if (::StringAt(::original, ::current + 1, 1, {"P","B"}))
            ::current += 2
         else
            ::current += 1
         endif
         ::primary   += "P"
         ::secondary += "P"

      case cChar == 'Q'
         if (substr(::original, ::current + 1, 1) == 'Q')
            ::current += 2
         else
            ::current += 1
         endif
         ::primary   += "K"
         ::secondary += "K"

      case cChar == 'R'
         // french e.g. 'rogier', but exclude 'hochmeier'
         if ((::current == ::last) ;
               .AND. !::SlavoGermanic(::original) ;
               .AND. ::StringAt(::original, ::current - 2, 2, {"IE"}) ;
               .AND. !::StringAt(::original, ::current - 4, 2, {"ME","MA"}))
            ::primary   += ""
            ::secondary += "R"
         else
            ::primary   += "R"
            ::secondary += "R"
         endif
         if (substr(::original, ::current + 1, 1) == 'R')
            ::current += 2
         else
            ::current += 1
         endif

      case cChar == 'S'
         // special cases 'island', 'isle', 'carlisle', 'carlysle'
         if (::StringAt(::original, ::current - 1, 3, {"ISL","YSL"}))
            ::current += 1
            LOOP
         endif
         // special case 'sugar-'
         if ((::current == 1) ;
               .AND. ::StringAt(::original, ::current, 5, {"SUGAR"}))
            ::primary   += "X"
            ::secondary += "S"
            ::current += 1
            LOOP
         endif

         if (::StringAt(::original, ::current, 2, {"SH"}))
            // germanic
            if (::StringAt(::original, ::current + 1, 4, {"HEIM","HOEK","HOLM","HOLZ"}))
               ::primary   += "S"
               ::secondary += "S"
            else
               ::primary   += "X"
               ::secondary += "X"
            endif
            ::current += 2
            LOOP
         endif

         // italian & armenian
         if (::StringAt(::original, ::current, 3, {"SIO","SIA"}) ;
               .OR. ::StringAt(::original, ::current, 4, {"SIAN"}))
            if (!::SlavoGermanic(::original))
               ::primary   += "S"
               ::secondary += "X"
            else
               ::primary   += "S"
               ::secondary += "S"
            endif
            ::current += 3
            LOOP
         endif

         // german & anglicisations, e.g. 'smith' match 'schmidt', 'snider' match 'schneider'
         // also, -sz- in slavic language altho in hungarian it is pronounced 's'
         if (((::current == 1) ;
               .AND.  ::StringAt(::original, ::current + 1, 1, {"M","N","L","W"})) ;
               .OR. ::StringAt(::original, ::current + 1, 1, {"Z"}))
            ::primary   += "S"
            ::secondary += "X"
            if (::StringAt(::original, ::current + 1, 1, {"Z"}))
               ::current += 2
            else
               ::current += 1
            endif
            LOOP
         endif

         if (::StringAt(::original, ::current, 2, {"SC"}))
            // Schlesinger's rule
            if (substr(::original, ::current + 2, 1) == 'H')
               // dutch origin, e.g. 'school', 'schooner'
               if (::StringAt(::original, ::current + 3, 2, {"OO","ER","EN","UY","ED","EM"}))
                  // 'schermerhorn', 'schenker'
                  if (::StringAt(::original, ::current + 3, 2, {"ER","EN"}))
                     ::primary   += "X"
                     ::secondary += "SK"
                  else
                     ::primary   += "SK"
                     ::secondary += "SK"
                  endif
                  ::current += 3
                  LOOP
               else
                  if ((::current == 1) ;
                        .AND. !::IsVowel(::original, 3) ;
                        .AND. (substr(::original, ::current + 3, 1) != 'W'))
                     ::primary   += "X"
                     ::secondary += "S"
                  else
                     ::primary   += "X"
                     ::secondary += "X"
                  endif
                  ::current += 3
                  LOOP
               endif

               if (::StringAt(::original, ::current + 2, 1, {"I","E","Y"}))
                  ::primary   += "S"
                  ::secondary += "S"
                  ::current += 3
                  LOOP
               endif
            endif
            // else
            ::primary   += "SK"
            ::secondary += "SK"
            ::current += 3
            LOOP
         endif

         // french e.g. 'resnais', 'artois'
         if ((::current == ::last) ;
               .AND. ::StringAt(::original, ::current - 2, 2, {"AI","OI"}))
            ::primary   += ""
            ::secondary += "S"
         else
            ::primary   += "S"
            ::secondary += "S"
         endif

         if (::StringAt(::original, ::current + 1, 1, {"S","Z"}))
            ::current += 2
         else
            ::current += 1
         endif

      case cChar == 'T'
         if (::StringAt(::original, ::current, 4, {"TION"}))
            ::primary   += "X"
            ::secondary += "X"
            ::current += 3
            LOOP
         endif

         if (::StringAt(::original, ::current, 3, {"TIA","TCH"}))
            ::primary   += "X"
            ::secondary += "X"
            ::current += 3
            LOOP
         endif

         if (::StringAt(::original, ::current, 2, {"TH"}) ;
               .OR. ::StringAt(::original, ::current, 3, {"TTH"}))
            // special case 'thomas', 'thames' or germanic
            if (::StringAt(::original, ::current + 2, 2, {"OM","AM"}) ;
                  .OR. ::StringAt(::original, 1, 4, {"VAN ","VON "}) ;
                  .OR. ::StringAt(::original, 1, 3, {"SCH"}))
               ::primary   += "T"
               ::secondary += "T"
            else
               ::primary   += "0"
               ::secondary += "T"
            endif
            ::current += 2
            LOOP
         endif

         if (::StringAt(::original, ::current + 1, 1, {"T","D"}))
            ::current += 2
         else
            ::current += 1
         endif
         ::primary   += "T"
         ::secondary += "T"

      case cChar == 'V'
         if (substr(::original, ::current + 1, 1) == 'V')
            ::current += 2
         else
            ::current += 1
         endif
         ::primary   += "F"
         ::secondary += "F"

      case cChar == 'W'
         // can also be in middle of word
         if (::StringAt(::original, ::current, 2, {"WR"}))
            ::primary   += "R"
            ::secondary += "R"
            ::current += 2
            LOOP
         endif

         if ((::current == 1) ;
               .AND. (::IsVowel(::original, ::current + 1) ;
               .OR. ::StringAt(::original, ::current, 2, {"WH"})))
            // Wasserman should match Vasserman
            if (::IsVowel(::original, ::current + 1))
               ::primary   += "A"
               ::secondary += "F"
            else
               // need Uomo to match Womo
               ::primary   += "A"
               ::secondary += "A"
            endif
         endif

         // Arnow should match Arnoff
         if (((::current == ::last) ;
               .AND. ::IsVowel(::original, ::current - 1)) ;
               .OR. ::StringAt(::original, ::current - 1, 5, {"EWSKI","EWSKY","OWSKI","OWSKY"}) ;
               .OR. ::StringAt(::original, 1, 3, {"SCH"}))
            ::primary   += ""
            ::secondary += "F"
            ::current += 1
            LOOP
         endif

         // polish e.g. 'filipowicz'
         if (::StringAt(::original, ::current, 4, {"WICZ","WITZ"}))
            ::primary   += "TS"
            ::secondary += "FX"
            ::current += 4
            LOOP
         endif

         // else skip it
         ::current += 1

      case cChar == 'X'
         // french e.g. breaux
         if (!((::current == ::last) ;
               .AND. (::StringAt(::original, ::current - 3, 3, {"IAU", "EAU"}) ;
               .OR. ::StringAt(::original, ::current - 2, 2, {"AU", "OU"}))))
            ::primary   += "KS"
            ::secondary += "KS"
         endif

         if (::StringAt(::original, ::current + 1, 1, {"C","X"}))
            ::current += 2
         else
            ::current += 1
         endif

      case cChar == 'Z'
         // chinese pinyin e.g. 'zhao'
         if (substr(::original, ::current + 1, 1) == "H")
            ::primary   += "J"
            ::secondary += "J"
            ::current += 2
            LOOP
         elseif (::StringAt(::original, ::current + 1, 2, {"ZO", "ZI", "ZA"}) ;
               .OR. (::SlavoGermanic(::original) ;
               .AND. ((::current > 1) ;
               .AND. substr(::original, ::current - 1, 1) != 'T')))
            ::primary   += "S"
            ::secondary += "TS"
         else
            ::primary   += "S"
            ::secondary += "S"
         endif

         if (substr(::original, ::current + 1, 1) == 'Z')
            ::current += 2
         else
            ::current += 1
         endif

      OTHERWISE

         ::current ++

      ENDCASE

   ENDDO

   ::primary   = substr(::primary,   1, 4)
   ::secondary = substr(::secondary, 1, 4)

return { ::primary, ::secondary }

* ---------------

METHOD DoubleMetaPhone:StringAt( cString, nStart, nLength, aList )

   LOCAL i

   IF nStart < 1 .OR. nStart > Len(cString)
      RETURN .F.
   ENDIF

   FOR i := 1 TO Len(aList)
      IF aList[i] == SubStr(cString,nStart,nLength)
         RETURN .T.
      ENDIF
   NEXT
RETURN .F.

* ---------------

METHOD DoubleMetaPhone:IsVowel( cString, nPos )

RETURN SubStr(cString,nPos,1) $ 'AEIOUY'

* ---------------

METHOD DoubleMetaPhone:SlavoGermanic(cString)

RETURN 'W' $ cString .OR. 'K' $ cString .OR. 'CZ' $ cString .OR. 'WITZ' $ cString


